function [BestParticle, BestParticleObj, RunData]=PSwarm(Problem, ...
    InitialPopulation, Options, varargin)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Particle swarm pattern search algorithm for global optimization.
%
% PSwarm algorithm takes advantage of the particle swarm algorithm in
% looking for a global optimum in the search phase of the pattern search
% algorithm. The pattern search enforces convergence for a local optimum.
%
%
% The pswarm MATLAB function syntax is
% [BestParticle, BestParticleObj, RunData] = PSwarm(Problem,
%                                  InitialPopulation, Options, P1, P2, ...)
%
% Input arguments:
%
%   Problem is a structure with some problem definitions
%
%     Problem.Variables        - Number of continuum real variables. If not
%                                specified then the size of the LB vector
%                                will be assigned to it.
%            .ObjFunction      - The name of the function to be called to
%                                when a objective function value is
%                                requested. This function syntax is
%                                [f]=myobj(x), where is a output argument
%                                - the objective function value - computed
%                                at (x), being x a vector with real
%                                variables.
%
%            .LB and .UB       - are column vector with the lower and upper
%                                bounds on the variables.
%
%            .A and .b         - Linear constraints Ax<=b
%
%   InitialPopulation is an array of structures.
%
%     InitialPopulation(1).x  -  Real variables initial guess to include in
%                                the initial population.
%                      (2).x  -  Second approximation for real ...
%       If the size of the structure array exceeds the population size then
%       the latter one will automatically be incremented.
%
%   Option is a structure with options to be passed to the algorithm. Call
%       pswarm('defaults') to obtain a list of the default options.
%
%   Just type opt=PSwarm('defaults') to obtain the default options for the
%       algorithm
%
%   P1, P2, ... are extra parameters passed to the objective evaluation
%       function
%
%
%
%
% Output arguments:
%
%   BestParticle              - The best particle obtained for the
%                               population (Leader)
%   BestParticleObj           - The best particle objective function
%                               obtained for the population (Leader
%                               objective function value)
%   RunData                   - Some statistics about the algorithm run
%
%
% Copyright (C) 2007 A. Ismael F. Vaz and L. N. Vicente.
%
% http://www.norg.uminho.pt/aivaz
% http://www.mat.uc.pt/~lnv
%
% This library is free software; you can redistribute it and/or
% modify it under the terms of the GNU Lesser General Public
% License as published by the Free Software Foundation; either
% version 2.1 of the License, or (at your option) any later version.
%
% This library is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% Lesser General Public License for more details.
%
% You should have received a copy of the GNU Lesser General Public
% License along with this library; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% aivaz@dps.uminho.pt 04/04/2007


% Default options
%  Cognitial - cognitial parameter in the velocity equation
%  InerciaFinalWeight - The value of the inercia parameter in the velocity
%             at last iteration.
%  InerciaInitialWeight - The value of the inercia parameter in the
%             velocity equation at first iteration.
%  MaxObj - Maximum number of objective function evaluations. Since the
%             algorithm is population based this maximum number of
%             objective function evaluation may be sligtly exceeded.
%  MaxIter - Maximum number of iterations allowed.
%  Size - Population size.
%  Social - Social parameter in the velocity equation.
%  MaxVelocityFactor - Velocity will be projected in the set
%             (UB-LB)*MaxVelocityFactor.
%  CPTolerance - Stopping criteria tolerance.
%  InitialDelta - Initial pattern search grid step.
%  DeltaIncreseFactor - Delta will be increased by this factor (on
%             successful poll steps).
%  DeltaDecreaseFactor - Delta willbe decreased by this factor (on
%             unsuccessful poll steps).
%  InitialDeltaFactor - The inicial Delta will be the min(UB-LB) times this
%             factor.
%  DegTolerance - Degenerancy tolerance.
%  LinearStepSize - Projection type on search step (particle swarm). =0 no
%               Step Size used on velocity . =1 Maximum Step Size if
%               computed. >1 Maximum Step Size if computed and velocity is
%               truncated for linear constraints.
%  EpsilonActive - Precision used to compute linear epsilon active
%               constraints.
%  TangentCone - 0 SID-PSM like version. 1 NOMAD like version. 2 SID-PSM
%               adapted (active constraints changes). 3 NullSpace version.
%               4 Simple QR factorization.
%
%  Trials - number of trials in generating initial guess.
%  IPrint - Iteration Print. If <0 no print is done. If =0 print the best
%           point found. If =n, n>0, print the best objective function
%           value each n iterations.
%  OutputFCN - Function to be called each IPrint iterations.
%  Vectorized - Call objective function in a vectorized way (=1) or point
%           by point (=0). In the vectorized version the poll step is
%           non-oportunistic, while in the not vectorized version the poll
%           step is oportunistic.
%
%
DefaultOpt = struct('Cognitial', 0.5, 'InerciaFinalWeight', 0.4,...
    'InerciaInitialWeight', 0.9, ...
    'MaxObj', 2000, 'MaxIter', 2000, 'Size', 20, 'Social', 0.5, ...
    'MaxVelocityFactor', 0.5, 'CPTolerance', 1.0e-5, 'InitialDelta', 20.0, ...
    'DeltaIncreaseFactor', 2.0, 'DeltaDecreaseFactor', 0.5, ...
    'InitialDeltaFactor', 5.0, 'IPrint', 10, 'DegTolerance', 0.001, ...
    'LinearStepSize', 1, 'EpsilonActive', 1e-1, 'TangentCone', 4,...
    'Trials', 1000, 'MVETrials', 10, 'OutputFCN', 'outputfcn', 'Vectorized', 0);

% With no arguments just print an error
if nargin==0 && nargout==0
    error('PSwarm:Arguments', 'Invalid number of arguments. Type ''help PSwarm'' to obtain help.');
end


% If just 'defaults' passed in, return the default options in X
if nargin==1 && nargout <= 1 && isequal(Problem,'defaults')
   BestParticle = DefaultOpt;
   return
end


% Check parameters consistence
if nargin < 1
  error('pswarm:AtLeastOneInput','PSwarm requests at least one imput (Problem definition).');
end

% If parameters are missing just define them as empty
if nargin < 3, Options=[];
    if nargin < 2, InitialPopulation = [];
    end
end

% Problem should be a structure
if ~isstruct(Problem)
    error('pswarm:StructProblem', 'First parameter must be a struct.');
end


% Do some sanity checkup on the user provided parameters and data
% We need an objective function
if ~isfield(Problem,'ObjFunction') || isempty(Problem.ObjFunction)
    error('pswarm:ObjMissing', 'Objective function name is missing.');
end

% and simple bound for particle swarm
if ~isfield(Problem,'LB') || ~isfield(Problem,'UB') || ~isnumeric(Problem.LB) || ~isnumeric(Problem.UB) || ...
        isempty(Problem.LB) || isempty(Problem.UB)
    error('pswarm:Bounds', 'Particle swarm relies on bounds on all variables.');
end

% Bound arrays must have the same size
if length(Problem.LB)~=length(Problem.UB)
    error('pswarm:BoundsSize', 'Lower bound and upper bound arrays length mismatch.');
end

% Compute the number of variables
% If the user provides these number check for correctness, otherwise try to
% compute them from the size of the bound constraints
if (~isfield(Problem, 'Variables') || isempty(Problem.Variables))
    % default is to consider all variables as real
    Problem.Variables=length(Problem.LB);
end

% Check for computed number of variables
if Problem.Variables<0 || Problem.Variables>length(Problem.LB)
    error('pswarm:VariablesNumber', 'Number of variables do not agree with bound constraints');
end

% Be sure to have a column vector
ULBSize=size(Problem.LB);
if(ULBSize(1)~=Problem.Variables)
    Problem.LB=Problem.LB';
end
ULBSize=size(Problem.UB);
if(ULBSize(1)~=Problem.Variables)
    Problem.UB=Problem.UB';
end




% Do some checkup in the linear constraints if they exist
if isfield(Problem, 'A')
    if isempty(Problem.A)
        Problem.mLinear=0;
    else
        [Problem.mLinear, nVars]=size(Problem.A);
        if Problem.Variables ~= nVars;
            error('pswarm:Linear', 'Number of variables for Linear constraints do not agree.');
        end
    end

    % Check for independent term
    if ~isfield(Problem, 'b') || Problem.mLinear ~= length(Problem.b)
        error('pswarm:IndependentTerm', 'Size of Linear independent term does not agree or is not defined.');
    end
else
    % Always define linear constraints
    Problem.mLinear=0;
    Problem.A=[];
    Problem.b=[];
end


% Start clock
tic;


% Initialize options. GetOption returns the user specified value, if the
% option exists. Otherwise returns the default option.

% These are local options. Not needed for the subrotines
MaxIterations=GetOption('MaxIter',Options,DefaultOpt);
MaxEvals=GetOption('MaxObj',Options,DefaultOpt);
InerciaInitial=GetOption('InerciaInitialWeight',Options,DefaultOpt);
InerciaFinal=GetOption('InerciaFinalWeight',Options,DefaultOpt);
Cognitial=GetOption('Cognitial',Options,DefaultOpt);
Social=GetOption('Social',Options,DefaultOpt);


% These are global options. Need to be available to subrotines.
Problem.IPrint=GetOption('IPrint', Options, DefaultOpt);
Problem.OutputFCN=GetOption('OutputFCN', Options, DefaultOpt);
Problem.IncreaseDelta=GetOption('DeltaIncreaseFactor',Options,DefaultOpt);
Problem.DecreaseDelta=GetOption('DeltaDecreaseFactor',Options,DefaultOpt);
Problem.DegTolerance=GetOption('DegTolerance',Options,DefaultOpt);
Problem.Tolerance=GetOption('CPTolerance', Options, DefaultOpt);
Problem.LinearStepSize=GetOption('LinearStepSize', Options, DefaultOpt);
Problem.EpsilonActive=GetOption('EpsilonActive', Options, DefaultOpt);
Problem.TangentCone=GetOption('TangentCone', Options, DefaultOpt);
Problem.Trials=GetOption('Trials', Options, DefaultOpt);
Problem.MVETrials=GetOption('MVETrials', Options, DefaultOpt);
Problem.Vectorized=GetOption('Vectorized', Options, DefaultOpt);


% Compute minimum delta for pattern search
% The minimum delta available should be at least twice the Tolerance
I=intersect(find(Problem.UB<Inf),find(Problem.LB>-Inf));
if ~isempty(I)
    InitialDelta=min(Problem.UB(I)-Problem.LB(I));
    InitialDelta = InitialDelta / GetOption('InitialDeltaFactor', ...
        Options, DefaultOpt);
else
    InitialDelta = GetOption('InitialDelta', Options, DefaultOpt);
end
if InitialDelta < sqrt(2*Problem.Tolerance)
    InitialDelta = sqrt(sqrt(Problem.Tolerance));
end

% Do not overwrite the user setup for InitialDelta
if ~isfield(Problem,'InitialDelta') || isempty(Problem.InitialDelta)
    Problem.InitialDelta=InitialDelta;
end


% compute the maximum allowed velocity. High velocities will be "pushing"
% particle to the border.
MaxVelocityVect=(Problem.UB(1:Problem.Variables)-Problem.LB(1:Problem.Variables))*...
    GetOption('MaxVelocityFactor',Options,DefaultOpt);

% compute variables scaling
FiniteVars=find(isfinite(Problem.LB.*Problem.UB));
FiniteConst=find(isfinite(Problem.b));

Problem.Scale=1;
if ~isempty(FiniteVars)
    Problem.Scale = max([max(abs(Problem.UB(FiniteVars)-Problem.LB(FiniteVars))), Problem.Scale]);
end
if ~isempty(FiniteConst)
    Problem.Scale = max([max(abs(Problem.b(FiniteConst))), Problem.Scale]);
end


if Problem.IPrint>=0
    disp('Initial delta for pattern search ');
    disp(Problem.InitialDelta);
end


% Initialize counters
% Iteration counter
Problem.Stats.IterCounter=0;
% How many consecutive iterations without success (progress in the leader
% particle).
IterUnsuccess=0;
% Maximum velcity among all perticles velocity
MaxVelocity=+Inf;


% Initialize statistics counters

% Number of objective function calls. This is the number of calls to
% Problem.ObjFunction. Since particle swarm relies in a infinite penalty
% function strategy the penalty number of evaluation may be different.
Problem.Stats.ObjFunCounter=0;

% Number of Poll steps taken
Problem.Stats.PollSteps=0;

% How many poll step had success
Problem.Stats.SuccPollSteps=0;

% How many times Degeneracy happens in poll step
Problem.Stats.Degenerate=0;

% Keep track of what direction had success in the last poll step. The Delta
% parameter is incremented only of a success occours twice in the same
% direction.
Problem.Poll.LastSuccess=[];

% User was already warned of unfeasible leader?
Problem.FeasWarning=0;


% Generate initial population. Include initial population provided by user
%  if available
[Problem,Population]=InitPopulation(Problem, InitialPopulation, ...
    GetOption('Size', Options, DefaultOpt), varargin{:});

% Delta for the pattern search.
Population.Delta=Problem.InitialDelta;

% Initialize patter search. Initialize the coordinate search directions.
Problem=InitPatternSearch(Problem);

% Main cycle of the algorithm
% Run pattern search until no success is attained. Call for a poll step in
% the leader particle whenever no success is recorded.

% Stop if the maximum number of iterations or objective function
% evaluations is reached.
while(Problem.Stats.IterCounter<MaxIterations && Problem.Stats.ObjFunCounter<MaxEvals)
    
    if Problem.IPrint>0 && ...
        (Problem.Stats.IterCounter==1 || mod(Problem.Stats.IterCounter,Problem.IPrint)==0)
            res=feval(Problem.OutputFCN, Problem, Population);
            if(isnumeric(res) && res<0)
                if Problem.IPrint>=0
                    disp('Stopping due to velocity and tolerance');
                end
                break;
            end
    end
    
    
%      if Problem.Stats.IterCounter <5 || (mod(Problem.Stats.IterCounter,10)==0)
%          hold off;
%          [x1,x2]=meshgrid(-3:.1:1,-2.5:.1:2.5); %Treccani
%          %[x1,x2]=meshgrid(-5:.1:5,-5:.1:5); % linear
%          %[x1,x2]=meshgrid(-1:.05:1,-1:.05:1); % cm2
%          contour(x1,x2,x1.^4 + 4*x1.^3 + 3.5*x1.^2 + x2.^2,65); % Treccani
%          %contour(x1,x2,0.1*(cos(5*pi*x1))+0.1*(cos(5*pi*x2)-x1.^2-x2.^2),20); % cm2
%          %contour(x1,x2,x1.^2+4*x2.^2,60); % linear
%          PlotPopulation(Problem, Population);
%          title(strcat('Iter=',int2str(Problem.Stats.IterCounter),...
%              '   Poll=',int2str(Problem.Stats.PollSteps),...
%              '   SuccPoll=',int2str(Problem.Stats.SuccPollSteps),...
%              '   Alpha=',num2str(Population.Delta),...
%              '   Nof=',int2str(Problem.Stats.ObjFunCounter),...
%              '   Of=', num2str(Population.fy(Population.Leader))));
%          exportfig(1, strcat('particle_treccani_',int2str(Problem.Stats.IterCounter),'.eps'),'color','rgb');
          
%      end
    
    
    % Stop also if particle swarm velocity is bellow Tolerance and pattern
    % search Delta is bellow Tolerance
    if MaxVelocity<Problem.Tolerance && Population.Delta<Problem.Tolerance
        if Problem.IPrint>=0
            disp('Stopping due to velocity and tolerance');
        end
        break;
    end
    
    % Stop if the number of active particles is equal to 1 and pattern
    % search Tolerance was attained.
    if Population.ActiveParticles <=1 && Population.Delta<Problem.Tolerance
        if Problem.IPrint>=0
            disp('Stopping due to single particle and tolerance');
        end
        break;
    end
    
    % Increment iteration counter.    
    Problem.Stats.IterCounter=Problem.Stats.IterCounter+1;

    % No success iteration at begining
    Success=false;
    
    
    if(Problem.Vectorized) 
        % Call the penalty function for all active particles in a vectorized way
        % This is just a question of memory storage, as PenaltyEval can
        % manage a vector of points

        % Get all active particles in the swarm
        ActiveParticlesIdx=find(Population.Active(:)==1);
        ActiveParticles=Population.x(ActiveParticlesIdx,:);
        [Problem,ObjValue]=...
            PenaltyEval(Problem, ActiveParticles, varargin{:});
        for i=1:length(ActiveParticlesIdx)
                % Was progress attained for the current particle?
                if Population.fy(ActiveParticlesIdx(i))>ObjValue(i)
                    % Yes. Update best particle position
                    Population.fy(ActiveParticlesIdx(i))=ObjValue(i);
                    Population.y(ActiveParticlesIdx(i),:)=Population.x(ActiveParticlesIdx(i),:);

                    % Check if new leader is available
                    if Population.fy(Population.Leader)>Population.fy(ActiveParticlesIdx(i))...
                            || Population.Leader==ActiveParticlesIdx(i)
                        Population.Leader=ActiveParticlesIdx(i);
                        % Particle swarm iteration declared as successful
                        Success=true;
                        % Reset last success direction for pattern search
                        Problem.Poll.LastSuccess=[];
                    end
                end
        end
    else
        % For all particles in the swarm
        for i=1:Population.Size
            % If particle is active
            if (Population.Active(i))
                % Compute Objective function value
                [Problem,ObjValue]=...
                    PenaltyEval(Problem, Population.x(i,:), varargin{:});
                % Was progress attained for the current particle?
                if Population.fy(i)>ObjValue
                    % Yes. Update best particle position
                    Population.fy(i)=ObjValue;
                    Population.y(i,:)=Population.x(i,:);

                    % Check if new leader is available
                    if Population.fy(Population.Leader)>Population.fy(i) || Population.Leader==i
                        Population.Leader=i;
                        % Particle swarm iteration declared as successful
                        Success=true;
                        % Reset last success direction for pattern search
                        Problem.Poll.LastSuccess=[];
                    end
                end
            end
        end
    end
    
    % Successful iteration?
    if ~Success
        % No success in search phase. Proceed to a poll step if possible.
        if Population.Delta >= Problem.Tolerance
            [Problem,Population]=PollStep(Problem,Population, ...
                varargin{:});
            Problem.Stats.PollSteps=Problem.Stats.PollSteps+1;
            % Reset on the number of unsuccessful iterations without a poll
            % step
            IterUnsuccess=0;
        else
            % An unsuccessful iteration without poll step
            IterUnsuccess=IterUnsuccess+1;
        end
    else
        % Success
        IterUnsuccess=0;
        % Leader changed.
        % Increase Delta. Reset on the local search.
        if Population.Delta<Problem.InitialDelta
            Population.Delta=Population.Delta*Problem.IncreaseDelta;
        end
        % check for lower bounds
        if Population.Delta<Problem.Tolerance
            Population.Delta=2*Problem.Tolerance;
        end
    end
    
    
    % Compute inercia.
    Inercia = InerciaInitial - ...
        (InerciaInitial-InerciaFinal)*Problem.Stats.IterCounter/MaxIterations;

    % Update velocity and new particle positions
    % For all particles
    for i=1:Population.Size
        % Is active?
        if Population.Active(i)
            % Update velocity for real variables
            Population.vx(i,:)=Projection((Inercia*Population.vx(i,:)+ ...
                Cognitial*unifrnd(0,ones(1,Problem.Variables)).*...
                  (Population.y(i,:)-Population.x(i,:))+...
                Social*unifrnd(0,ones(1,Problem.Variables)).*...
                  (Population.y(Population.Leader,:)-Population.x(i,:)))',...
                -MaxVelocityVect,MaxVelocityVect);
            % Update particle position and check bound limits
            

            AlphaMax=ones(Problem.Variables,1);
            % check for simple bound
            I=find(Population.vx(i,:)<0); % we could violate lower bounds
            if ~isempty(I)
                AlphaMax(I)=...
                    min(AlphaMax(I),(Problem.LB(I)-Population.x(i,I)')...
                    ./Population.vx(i,I)');
            end
            I=find(Population.vx(i,:)>0); % we could violate upper bounds
            if ~isempty(I)
                AlphaMax(I)=...
                    min(AlphaMax(I),(Problem.UB(I)-Population.x(i,I)')...
                    ./Population.vx(i,I)');
            end

            AlphaMax=max(zeros(Problem.Variables,1),AlphaMax);
            
            velocity=AlphaMax.*Population.vx(i,:)';
            
            AlphaMaxLinCons=1.0;
            if Problem.mLinear>0 && Problem.LinearStepSize
                I=find(Problem.A*velocity>0);
                if ~isempty(I)
                    % We are not allowing a step longer than 1
                    AlphaMaxLinCons=...
                        min(AlphaMaxLinCons,min((Problem.b(I)-Problem.A(I,:)*Population.x(i,:)')...
                        ./(Problem.A(I,:)*velocity)));
                end
            end

            if(AlphaMaxLinCons>0)
                
                Population.x(i,:)=Projection(Population.x(i,:)'+(AlphaMaxLinCons.*velocity),...
                    Problem.LB(1:Problem.Variables), ...
                    Problem.UB(1:Problem.Variables));

                if Problem.mLinear>0 && Problem.LinearStepSize>1 % truncate velocity
                    Population.vx(i,:)=(AlphaMaxLinCons.*velocity)';
                end
            end
        end
    end

    % To compute population norm. Start with Leader.
    MaxVelocity=norm(Population.x(Population.Leader));
    
    % Reset number of active particles.
    Population.ActiveParticles=0;
    for i=1:Population.Size
        % Check if particle is active and we do not want to remove the
        % leader.
        if Population.Active(i) && Population.Leader~=i
            % compute particle velocity norm (for the stopping criteria.
            VelocityNorm=norm(Population.vx(i,:));
            Distance=norm(Population.x(i,:)-Population.x(Population.Leader,:));
            if Distance<Population.Delta && VelocityNorm<Population.Delta
                % Is neighbour
                Population.Active(i)=false;
            else
                MaxVelocity=MaxVelocity+VelocityNorm;
            end
        end
        % Account for active particles.
        if Population.Active(i)
            Population.ActiveParticles=Population.ActiveParticles+1;
        end
    end
    
    

                    
end


% End of main cycle ...


% hold off;
% [x1,x2]=meshgrid(-3:.1:1,-2.5:.1:2.5); %Treccani
% %[x1,x2]=meshgrid(-5:.1:5,-5:.1:5); % linear
% %[x1,x2]=meshgrid(-1:.05:1,-1:.05:1); % cm2
% contour(x1,x2,x1.^4 + 4*x1.^3 + 3.5*x1.^2 + x2.^2,65); % Treccani
% %contour(x1,x2,0.1*(cos(5*pi*x1))+0.1*(cos(5*pi*x2)-x1.^2-x2.^2),20); % cm2
% %contour(x1,x2,x1.^2+4*x2.^2,60); % linear
% PlotPopulation(Problem, Population);
% title(strcat('Iter=',int2str(Problem.Stats.IterCounter),...
%     '   Poll=',int2str(Problem.Stats.PollSteps),...
%     '   SuccPoll=',int2str(Problem.Stats.SuccPollSteps),...
%     '   Alpha=',num2str(Population.Delta),...
%     '   Nof=',int2str(Problem.Stats.ObjFunCounter),...
%     '   Of=', num2str(Population.fy(Population.Leader))));
% exportfig(1, strcat('particle_treccani_',int2str(Problem.Stats.IterCounter),'.eps'),'color','rgb');


% print final time
if Problem.IPrint>=0
    toc;
end

% Print if it was stopped due to the maximum of iterations or objective
% function evaluations
if Problem.Stats.IterCounter>=MaxIterations || Problem.Stats.ObjFunCounter>=MaxEvals
    if Problem.IPrint>=0
        disp('Maximum number of iterations or objective function evaluations reached');
    end
end

% Print for feasibility of the leader
if Problem.IPrint>=0 && Problem.mLinear>0
    if max(Problem.A*Population.y(Population.Leader,:)'-Problem.b)<=eps
        fprintf('Best particle is linear feasible\n');
    else
        error('Best particle is NOT linear feasible');
    end
end

% return leader position and objective function value
BestParticle=[Population.y(Population.Leader,:)]';
BestParticleObj=Population.fy(Population.Leader);
RunData=Problem.Stats;

if Problem.IPrint>=0
    % display some statistics
    disp(Problem.Stats);
end

return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% subrotine InitPopulation
%    Randomly initialize the population
%    Include initial guesses, if provided by the user
%
% Input:
%   Problem - problem data
%   InitialPopulation - Inicial population provided by user
%   Size - Requested size for the population
%
% Output:
%   Problem - problem data (problem data may be update)
%   Population - population data
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [Problem,Population]=InitPopulation(Problem, ...
    InitialPopulation, Size, varargin)

% Number of particles in initial population that were accepted
nP=0;

if Problem.IPrint>=0
    disp('Generating initial population');
end

% Check if user provides a valid initial population
if ~isempty(InitialPopulation) && ~isstruct(InitialPopulation)
    error('pswarm:InitPopulation:InitialPopulation', 'Initial population must be defined in a structure.');
else
    % Check for size
    if length(InitialPopulation)>Size
        % User provided an initial population greater than the population
        % size
        if Problem.IPrint>=0
            fprintf('Initial population is greater than population size.\nIncreasing population size from ');
            fprintf('%i',Size);
            fprintf(' to ');
            fprintf('%i',length(InitialPopulation));
        end
        % Population size is increased to fit the number of initial guesses
        Population.Size=length(InitialPopulation);
    else
        % Otherwise just accept the proposed population size
        Population.Size=Size;
    end
    
    % Copy the initial population for the population and initialize them
    % This is only done if the particle, after projection, is linear
    % feasible
    for i=1:length(InitialPopulation)
        % Particle position.
        Population.x(nP+1,:)=Projection(InitialPopulation(i).x,...
                Problem.LB(1:Problem.Variables), ...
                Problem.UB(1:Problem.Variables));
        if Problem.mLinear==0 || max(Problem.A*Population.x(nP+1,:)'-Problem.b)<=eps
            %particle is linear feasible
            nP=nP+1;
            
            % Best particle position.
            Population.y(nP,:)=Population.x(nP,:);
            % Particle velocities.
            Population.vx(nP,:)=zeros(1,Problem.Variables);
            % Particle is active at begining
            Population.Active(nP)=true;
            Population.fy(i)=+Inf;
%            [Problem,Population.fy(nP)]=+Inf;
%                PenaltyEval(Problem, Population.x(nP,:), varargin{:});
        end
    end
end

if (nP<length(InitialPopulation))
    warning('Only %d particles were linear feasible after projection', nP);
end

% Ramdomly generate the remaining population
% They must be linear feasible

Ellipsoid=0;

if Problem.mLinear > 0
    if Problem.IPrint>=0
        disp('Generating Ellipsoid');
    end
    
    Ellipsoid=1; % we are computing an ellipsoid
    
    [A,b]=RealBounds(Problem);
    
    % We have linear constraints, so take care of the polytope
    maxiter = 80; tol1 = 1.e-8; tol2 = 1.e-6;

    if (nP<=0) || max(Problem.A*Population.x(1,:)'-Problem.b)>=eps ...
            || max(Population.x(1,:)'-Problem.LB)>=eps ...
            || max(Problem.UB-Population.x(1,:)')>=eps
        % We do not have a linear feasible approximation
        % Get one from presolve
        [msg,x0] = mve_presolve(A, b, maxiter,...
            zeros(Problem.Variables,1), tol1, 0); % initial guess with zeros
        if msg(1) ~= 's'
            % error on presolver.
            if Problem.IPrint>=0
                disp('MVE presolver error');
                disp(msg);
                disp('Trying to recover by adding fictitious bounds');
            end
            [A,b]=FicticiousBounds(Problem,A,b);
            [msg,x0] = mve_presolve(A, b, maxiter,...
                zeros(Problem.Variables,1), tol1, 1);
            if msg(1) ~= 's'
                if Problem.IPrint>=0
                    disp('MVE presolver error');
                    disp(msg);
                end
                
                trial=0;
                [lb,ub]=GetBounds(Problem); % to randomly generate initial guess into bound constraints
                while msg(1)~='s' && trial < Problem.MVETrials
                    [msg,x0] = mve_presolve(A, b, maxiter, unifrnd(lb,ub),...
                        tol1, 0);
                    if msg(1) ~= 's' && Problem.IPrint>=0
                        disp('MVE presolver error');
                        disp(msg);
                    end
                    trial=trial+1;
                end
                if msg(1) ~= 's'
                    if Problem.IPrint>=0
                        disp('MVE presolver error');
                        disp(msg);
                        disp('We are leaving the ellipsoid strategy');
                    end
                    Ellipsoid=0;
                end
            end
        end
    else
        x0=Population.x(1,:)'; % First as initial guess
    end

    if(Ellipsoid==1)
        % Get the ellipsoid
        % use again real bounds
        [A,b]=RealBounds(Problem);
        
        [x,E2,msg] = mve_solver(A, b, x0, maxiter, tol2);
        
        if ~msg
            if Problem.IPrint>=0
                disp('MVE solver error');
                disp('Trying to recover by adding fictitious bounds');
            end
            
            [A,b]=FicticiousBounds(Problem,A,b);

            [x,E2,msg] = mve_solver(A, b, x0, maxiter, tol2);

            %if ~msg
            %    disp('MVE solver error');
            %    disp('We are leaving the ellipsoid strategy');
            %    Ellipsoid=0;
            %end
        end
       
        %if(msg)
            [E,out] = chol(E2);

            if(out>0)
                if Problem.IPrint>=0
                    disp('Unable obtain ellipsoid');
                end
                Ellipsoid=0;
            else
                E = E';
            end

            % x - elipsoid center
            % E - Elipsoid matrix

            %hold off;

            %draw_ellipse([Problem.A; eye(Problem.Variables);...
            %    -eye(Problem.Variables)],...
            %    [Problem.b; Problem.UB; -Problem.LB],x0,x,E,[]);
        %else
        %    disp('We are leaving the ellipsoid strategy');
        %    Ellipsoid=0;
        %end
    end
end % End of ellipsoid computation

if ~Ellipsoid % get real bounds
    [lb,ub]=GetBounds(Problem);
end
        
for i=nP+1:Population.Size
    % Particle positions.
    if Ellipsoid==0
        if Problem.mLinear>0
            Population.x(i,:)=GetFeasiblePoint(Problem,lb,ub);
        else
            Population.x(i,:)=unifrnd(lb,ub);
        end
    else
        tmppt=unifrnd(-ones(Problem.Variables,1), ones(Problem.Variables,1));
        tmppt=tmppt/sqrt(tmppt'*tmppt);
        Population.x(i,:) = Projection(x +...
            (unifrnd(0, 1))^(1/Problem.Variables)*E*tmppt,...
            Problem.LB(1:Problem.Variables), ...
            Problem.UB(1:Problem.Variables));
        if max(Problem.A*Population.x(i,:)'-Problem.b)>0
            error('Not feasible pop. Internal error. This should not happen.');
        end
    end
    % Best particle position.
    Population.y(i,:)=Population.x(i,:);
    % Particle velocities
    Population.vx(i,:)=zeros(1,Problem.Variables);
    % Particle active or inactive
    Population.Active(i)=true;
    Population.fy(i)=+Inf;
%    [Problem,Population.fy(i)]=...
%        PenaltyEval(Problem, Population.x(i,:), varargin{:});
end

Population.ActiveParticles=Population.Size;
Population.Leader=1;

% plot population
%PlotPopulation(Problem, Population);

% plot ellipsoid
%t=0:0.1:2*pi;
%ellipsoid=E*[cos(t);sin(t)];
%plot(x(1)+ellipsoid(1,:),x(2)+ellipsoid(2,:));
 
return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  subrotine GetOption
%
%  Input:
%    Option - option to get the value
%    Options - a list of options provided by user
%    DefaultOpt -  a list of default options
%
%  Output:
%    Value - The value specified by user for Option or the default
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [Value]=GetOption(Option, Options, DefaultOpt)

% Check for user provided options
%if isempty(Options) || ~isstruct(Options)
    % User does not provides Options
%    Value=DefaultOpt.(Option);
%    return;
%end

% Try the option provided by user
try
    Value=Options.(Option);
catch
    Value=[];
end

% Option not provided by user
if isempty(Value)
    Value=DefaultOpt.(Option);    
end

return


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Plot the population
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function []=PlotPopulation(Problem, Population)

if Problem.Variables ~= 2
    return;
end

hold on;

for j=1:Population.Size
    if Population.Active(j)
        plot(Population.y(j,1),Population.y(j,2),'ok');
    end
end
    
return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Compute real bounds
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [A,b]=RealBounds(Problem)

if Problem.mLinear>0
    % Construct the polytope with linear constraints
    Id=eye(Problem.Variables); % Identity matrix for bound constraints
    A=[Problem.A; Id(find(Problem.UB<Inf),:); -Id(find(Problem.LB>-Inf),:)];
    b=[Problem.b; Problem.UB(find(Problem.UB<Inf)); -Problem.LB(find(Problem.LB>-Inf))];
else
    % Construct the polytope without linear constraints
    Id=eye(Problem.Variables); % Identity matrix for bound constraints
    A=[Id(find(Problem.UB<Inf),:); -Id(find(Problem.LB>-Inf),:)];
    b=[Problem.UB(find(Problem.UB<Inf)); -Problem.LB(find(Problem.LB>-Inf))];
end
return;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Compute ficticious bounds
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [A,b]=FicticiousBounds(Problem,RealA,Realb)

A=RealA;
b=Realb;
Id=eye(Problem.Variables);

I=intersect(find(Problem.UB<Inf),find(Problem.LB<=-Inf));
% Upper bound finite and Lower bound not finite
if ~isempty(I)
    A=[A; -Id(I,:)];
    % Fictitious bound is three times the distance to origin
    o_I=ones(length(I),1);
    b=[b; -min(-100*o_I,Problem.UB(I)-3*abs(Problem.UB(I)))];
end
I=intersect(find(Problem.LB>-Inf), find(Problem.UB>=Inf));
% Lower bound finite and Upper bound not finite
if ~isempty(I)
    A=[A; Id(I,:)];
    o_I=ones(length(I),1);
    % Fictitious bound is three times the distance to origin
    b=[b; max(100*o_I,Problem.LB(I)+3*abs(Problem.LB(I)))];
end
I=intersect(find(Problem.UB>=Inf),find(Problem.LB<=-Inf)); % Both limites not finite
if ~isempty(I)
    A=[A; -Id(I,:); Id(I,:)];
    o_I=ones(length(I),1);
    b=[b; -o_I*min(-100, -10*min(Problem.LB>-Inf)); o_I*max(100, 10*max(Problem.UB<Inf))];
end

return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Compute bounds (add ficticious if necessary)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [lb,ub]=GetBounds(Problem)

lb=Problem.LB;
ub=Problem.UB;

I=intersect(find(Problem.UB<Inf),find(Problem.LB<=-Inf));
% Upper bound finite and Lower bound not finite
if ~isempty(I)
    o_I=ones(length(I),1);
    lb(I)=min(-100*o_I,-Problem.UB(I)+3*abs(Problem.UB(I)));
end
I=intersect(find(Problem.LB>-Inf), find(Problem.UB>=Inf));
% Lower bound finite and Upper bound not finite
if ~isempty(I)
    o_I=ones(length(I),1);
    ub(I)=max(100*o_I,Problem.LB(I)+3*abs(Problem.LB(I)));
end
I=intersect(find(Problem.UB>=Inf),find(Problem.LB<=-Inf)); % Both limites not finite
if ~isempty(I)
    o_I=ones(length(I),1);
    lb(I)=o_I*min(-100, 10*min(Problem.LB>-Inf));
    ub(I)=o_I*max(100, 10*max(Problem.UB<Inf));
end

return;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Get Feasible point by try and error
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [x]=GetFeasiblePoint(Problem,lb,ub)

for i=1:Problem.Trials
    x=unifrnd(lb,ub);
    if max(Problem.A*x-Problem.b)<=0
        % Is linear feasible
        return;
    end
end

disp('Failled to generate feasible initial guess');
error('Unable to proceed!');

return;